define(["require", "exports", "./_.contribution", "./fillers/monaco-editor-core"], function (require, exports, __contribution_1, monaco_editor_core_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.DiagnosticsAdapter = void 0;
    var DiagnosticsAdapter = /** @class */ (function () {
        function DiagnosticsAdapter(_languageId, _worker, defaults) {
            var _this = this;
            this._languageId = _languageId;
            this._worker = _worker;
            this._disposables = [];
            this._listener = Object.create(null);
            var onModelAdd = function (model) {
                var modeId = model.getLanguageId();
                if (modeId !== _this._languageId) {
                    return;
                }
                _this._listener[model.uri.toString()] = model.onDidChangeContent(__contribution_1.debounce(function () {
                    _this._doValidate(model.uri, modeId);
                }, 600));
                _this._doValidate(model.uri, modeId);
            };
            var onModelRemoved = function (model) {
                monaco_editor_core_1.editor.setModelMarkers(model, _this._languageId, []);
                var uriStr = model.uri.toString();
                var listener = _this._listener[uriStr];
                if (listener) {
                    listener.dispose();
                    delete _this._listener[uriStr];
                }
            };
            this._disposables.push(monaco_editor_core_1.editor.onDidCreateModel(onModelAdd));
            this._disposables.push(monaco_editor_core_1.editor.onWillDisposeModel(onModelRemoved));
            this._disposables.push(monaco_editor_core_1.editor.onDidChangeModelLanguage(function (event) {
                onModelRemoved(event.model);
                onModelAdd(event.model);
            }));
            defaults.onDidChange(function (_) {
                monaco_editor_core_1.editor.getModels().forEach(function (model) {
                    if (model.getLanguageId() === _this._languageId) {
                        onModelRemoved(model);
                        onModelAdd(model);
                    }
                });
            });
            this._disposables.push({
                dispose: function () {
                    for (var key in _this._listener) {
                        _this._listener[key].dispose();
                    }
                }
            });
            monaco_editor_core_1.editor.getModels().forEach(onModelAdd);
        }
        DiagnosticsAdapter.prototype.dispose = function () {
            this._disposables.forEach(function (d) { return d && d.dispose(); });
            this._disposables = [];
        };
        DiagnosticsAdapter.prototype._doValidate = function (resource, languageId) {
            this._worker(resource)
                .then(function (worker) {
                var _a;
                return worker.doValidation(((_a = monaco_editor_core_1.editor.getModel(resource)) === null || _a === void 0 ? void 0 : _a.getValue()) || '');
            })
                .then(function (diagnostics) {
                var markers = diagnostics.map(function (d) { return toDiagnostics(resource, d); });
                var model = monaco_editor_core_1.editor.getModel(resource);
                if (model && model.getLanguageId() === languageId) {
                    monaco_editor_core_1.editor.setModelMarkers(model, languageId, markers);
                }
            })
                .then(undefined, function (err) {
                console.error(err);
            });
        };
        return DiagnosticsAdapter;
    }());
    exports.DiagnosticsAdapter = DiagnosticsAdapter;
    function toSeverity(lsSeverity) {
        switch (lsSeverity) {
            default:
                return monaco_editor_core_1.MarkerSeverity.Error;
        }
    }
    function toDiagnostics(resource, diag) {
        var code = typeof diag.code === 'number' ? String(diag.code) : diag.code;
        return {
            severity: toSeverity(diag.severity),
            startLineNumber: diag.startLine,
            startColumn: diag.startCol,
            endLineNumber: diag.endLine,
            endColumn: diag.endCol,
            message: diag.message,
            code: code,
            source: diag.source
        };
    }
});
